/*
 ===========================================================================
 =                                                                         =
 = File:                                                                   =
 =   PASSWD_VSCAN.C                                                          =
 =                                                                         =
 = Synopsis:                                                               =
 =   This file contains functions that implement password checking.        =
 =                                                                         =
 = Authors:                                                                =
 =   Alberto Meregalli, DIF                                                =
 =                                                                         =
 ===========================================================================
*/

#include <starlet.h>
#include <ssdef.h>
#include <uaidef.h>
#include <rmsdef.h>
#include <jpidef.h>
#include <ciadef.h> 
#include <secsrvdef.h>

/* VMS 6.1 lacks the following: extracted from starlet and hand adjusted */
#include "secsrvmsgdef.h"

#include <lib$routines.h>

#include <descrip.h>
#include <string.h>
#include <stdio.h>

typedef struct {
  unsigned short buflen;
  unsigned short itemcode;
  char *bufadr;
  short *retlen;
} list_item;

/* additional prototypes needed */
void	system_log (int log_level, char *format, ...);
char	*vms_message(unsigned int code);
#define LOG_ERROR  0

/*
 * given username, password and source identification this routine
 *
 * - checks whether there is such a user
 * - checks whether the password is correct
 * - updates and checks the intrusion database
 * - only if everything is ok (i.e. the user exists, the password is the 
 *     right one AND no intrusion matching user and source is in progess)
 *    returns the value 1
 *   otherwise returns 0
 *
 */

int PWDcheck (username, password, source_terminal, source_node,
			source_user)

    char *username, *password, *source_terminal, *source_node, *source_user;

{
  struct {
    list_item item3[3];
    unsigned int zero;
  } itmlist ;

  struct dsc$descriptor_s desc_user, desc_passw, desc_sr_term,
			    desc_sr_node, desc_sr_user;

  short int lencrypt, lsalt, lpwdg, salt;
  int stato, pwdg[2],  hash[2];
  int log_fail, retval;
  unsigned char encrypt;

  

  itmlist.item3[0].buflen = 8;
  itmlist.item3[0].itemcode = UAI$_PWD;
  itmlist.item3[0].bufadr = (char *) pwdg;
  itmlist.item3[0].retlen = &lpwdg;

  itmlist.item3[1].buflen = 2;
  itmlist.item3[1].itemcode = UAI$_SALT;
  itmlist.item3[1].bufadr = (char *) &salt;
  itmlist.item3[1].retlen = &lsalt;

  itmlist.item3[2].buflen = 1;
  itmlist.item3[2].itemcode = UAI$_ENCRYPT;
  itmlist.item3[2].bufadr = (char *) &encrypt;
  itmlist.item3[2].retlen = &lencrypt;

  itmlist.zero = 0;

  desc_user.dsc$w_length = strlen (username);
  desc_user.dsc$a_pointer = username;
  desc_user.dsc$b_class = DSC$K_CLASS_S;
  desc_user.dsc$b_dtype = DSC$K_DTYPE_T;

  desc_sr_term.dsc$w_length = strlen (source_terminal);
  desc_sr_term.dsc$a_pointer = source_terminal;
  desc_sr_term.dsc$b_class = DSC$K_CLASS_S;
  desc_sr_term.dsc$b_dtype = DSC$K_DTYPE_T;

  desc_sr_node.dsc$w_length = strlen (source_node);
  desc_sr_node.dsc$a_pointer = source_node;
  desc_sr_node.dsc$b_class = DSC$K_CLASS_S;
  desc_sr_node.dsc$b_dtype = DSC$K_DTYPE_T;
            
  desc_sr_user.dsc$w_length = strlen (source_user);
  desc_sr_user.dsc$a_pointer = source_user;
  desc_sr_user.dsc$b_class = DSC$K_CLASS_S;
  desc_sr_user.dsc$b_dtype = DSC$K_DTYPE_T;

  desc_passw.dsc$w_length = strlen (password);
  desc_passw.dsc$a_pointer = password;
  desc_passw.dsc$b_class = DSC$K_CLASS_S;
  desc_passw.dsc$b_dtype = DSC$K_DTYPE_T;

  /* check if the user is known */
  if ((stato = sys$getuai (0, 0, &desc_user, &itmlist, 0, 0, 0)) == 
	RMS$_RNF) 
  {

    stato = sys$scan_intrusion (SS$_NOSUCHUSER, &desc_user, JPI$K_NETWORK, 
			&desc_sr_term, &desc_sr_node, &desc_sr_user,
			0, 0, 0, 0, CIA$M_IGNORE_RETURN);
    if (stato & 1 == 0) 
      system_log (LOG_ERROR,"sys$scan_intrusion: %s", vms_message(stato));
    return 0;
  }
  /* check the password */

  if ((stato = sys$hash_password (&desc_passw,
				  encrypt, salt, &desc_user, hash)) 
	& 1 == 0) 
    system_log (LOG_ERROR,"sys$hash_password: %s", vms_message(stato));
  
  if (hash[0] == pwdg[0] && hash[1] == pwdg[1])
  {
    /* password is OK, tell SCAN_INTRUSION and return the value corresponding
     * to its reply */
    if ((stato = sys$scan_intrusion (SS$_NORMAL, &desc_user,
				    JPI$K_NETWORK, 
				    &desc_sr_term, &desc_sr_node, 
				    &desc_sr_user, 0, 0, 0, 0, 0)) == 
		SECSRV$_INTRUDER)
         retval = 0;
    else retval = 1;
  }
  else
  {
    /* password is incorrect, tell SCAN_INTRUSION and return 0 */
    stato = sys$scan_intrusion (SS$_INVLOGIN, &desc_user,
				    JPI$K_NETWORK, 
				    &desc_sr_term, &desc_sr_node, 
				    &desc_sr_user, 0, 
				    &desc_passw, 0, 0,
				    CIA$M_REAL_USERNAME);
    retval = 0;
  }  
  /* signal any strange reply from SCAN_INTRUSION */
  if (stato != SECSRV$_INTRUDER &&
      stato != SECSRV$_SUSPECT &&
      stato != SS$_NORMAL &&
      stato != SECSRV$_NOSCANNEDINTRUDER ) 
    system_log (LOG_ERROR,"sys$scan_intrusion: %s", vms_message(stato));
  return retval;
}
